window.Gleam.initSearch([{"doc":"bath","title":"bath","content":"# 🛁 Bath\n\n[![Package Version](https://img.shields.io/hexpm/v/bath)](https://hex.pm/packages/bath)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/bath/)\n\nBath is a generic resource pool for Gleam. It can be used to manage a pool of\nany value, such as database connections, file handles, or other resources.\n\n## Installation\n\n```sh\ngleam add bath@2\n```\n\n## Usage\n\n```gleam\nimport bath\nimport fake_db\n\npub fn main() {\n  // Create a pool of 10 connections to some fictional database.\n  let assert Ok(pool) =\n    bath.new(fn() { fake_db.connect() })\n    |&gt; bath.with_size(10)\n    |&gt; bath.with_shutdown(fn(conn) { fake_db.close(conn) })\n    |&gt; bath.start(1000)\n\n  // Use the pool. Shown here in a block to use `use`.\n  let assert Ok(&quot;Hello!&quot;) = {\n    use conn &lt;- bath.apply(pool, 1000)\n    // Do stuff with the connection...\n    &quot;Hello!&quot;\n  }\n\n  // Close the pool.\n  let assert Ok(_) = bath.shutdown(pool, fn(conn) { fake_db.close(conn) }, 1000)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/bath&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"bath","title":"ApplyError","content":"pub type ApplyError(resource_create_error) {\n  NoResourcesAvailable\n  CheckOutResourceCreateError(resource_create_error)\n  CheckOutTimeout\n}\n An error returned when failing to apply a function to a pooled resource.\nNoResourcesAvailable\n\n\nCheckOutResourceCreateError(resource_create_error)\n\n\nCheckOutTimeout\n\n\nSynonyms:\nbath.ApplyError\nbath ApplyError","url":"bath.html#ApplyError"},{"doc":"bath","title":"CheckoutStrategy","content":"pub type CheckoutStrategy {\n  FIFO\n  LIFO\n}\n The strategy used to check out a resource from the pool.\nFIFO\n\n\nLIFO\n\n\nSynonyms:\nbath.CheckoutStrategy\nbath CheckoutStrategy","url":"bath.html#CheckoutStrategy"},{"doc":"bath","title":"CreationStrategy","content":"pub type CreationStrategy {\n  Lazy\n  Eager\n}\n How to create resources in the pool. `Lazy` will create resources when\n required (i.e. the pool is empty but has extra capacity), while `Eager` will\n create the maximum number of resources upfront.\nLazy\n\n\nEager\n\n\nSynonyms:\nbath.CreationStrategy\nbath CreationStrategy","url":"bath.html#CreationStrategy"},{"doc":"bath","title":"Msg","content":"pub opaque type Msg(resource_type, resource_create_error)\n A message sent to the pool actor.\n\nSynonyms:\nbath.Msg\nbath Msg","url":"bath.html#Msg"},{"doc":"bath","title":"Pool","content":"pub opaque type Pool(resource_type, resource_create_error)\n The interface for interacting with a pool of resources in Bath.\n\nSynonyms:\nbath.Pool\nbath Pool","url":"bath.html#Pool"},{"doc":"bath","title":"PoolConfig","content":"pub opaque type PoolConfig(resource_type, resource_create_error)\n Configuration for a [`Pool`](#Pool).\n\nSynonyms:\nbath.PoolConfig\nbath PoolConfig","url":"bath.html#PoolConfig"},{"doc":"bath","title":"ShutdownError","content":"pub type ShutdownError {\n  ResourcesInUse\n  ShutdownTimeout\n  CalleeDown(reason: dynamic.Dynamic)\n}\n An error returned when the resource pool fails to shut down.\nResourcesInUse\n There are still resources checked out. Ignore this failure case by\n calling [`shutdown`](#shutdown) function with `force` set to `True`.\n\nShutdownTimeout\n The shutdown timeout expired.\n\nCalleeDown(reason: dynamic.Dynamic)\n The pool was already down or failed to send the response message.\n\nSynonyms:\nbath.ShutdownError\nbath ShutdownError","url":"bath.html#ShutdownError"},{"doc":"bath","title":"StartError","content":"pub type StartError(resource_create_error) {\n  PoolStartResourceCreateError(resource_create_error)\n  ActorStartError(actor.StartError)\n}\n An error returned when creating a [`Pool`](#Pool).\nPoolStartResourceCreateError(resource_create_error)\n\n\nActorStartError(actor.StartError)\n\n\nSynonyms:\nbath.StartError\nbath StartError","url":"bath.html#StartError"},{"doc":"bath","title":"State","content":"pub opaque type State(resource_type, resource_create_error)\n Pool actor state.\n\nSynonyms:\nbath.State\nbath State","url":"bath.html#State"},{"doc":"bath","title":"apply","content":"pub fn apply(\n  pool pool: Pool(a, b),\n  timeout timeout: Int,\n  next next: fn(a) -&gt; c,\n) -&gt; Result(c, ApplyError(b))\n Check out a resource from the pool, apply the `next` function, then check\n the resource back in.\n\n \n let assert Ok(pool) =\n   bath.new(fn() { Ok(&quot;Some pooled resource&quot;) })\n   |&gt; bath.start(1000)\n\n use resource &lt;- bath.apply(pool, 1000)\n // Do stuff with resource...\n \nSynonyms:\nbath.apply\nbath apply","url":"bath.html#apply"},{"doc":"bath","title":"child_spec","content":"pub fn child_spec(\n  config pool_config: PoolConfig(a, b),\n  timeout init_timeout: Int,\n) -&gt; Result(Spec(State(a, b), Msg(a, b)), StartError(b))\n Create a child actor spec pool actor, for use in your application&#39;s supervision tree,\n using the given [`PoolConfig`](#PoolConfig). Once the pool is started, use\n [`from_subject`](#from_subject) to create a [`Pool`](#Pool) from the\n `Subject(bath.Msg(a, b))`.\nSynonyms:\nbath.child_spec\nbath child_spec","url":"bath.html#child_spec"},{"doc":"bath","title":"from_subject","content":"pub fn from_subject(\n  subject subject: Subject(Msg(a, b)),\n) -&gt; Pool(a, b)\n Create a [`Pool`](#Pool) from a `Subject(bath.Msg(a, b))`. Useful when\n creating a pool as part of a supervision tree via the\n [`child_spec`](#child_spec) function.\nSynonyms:\nbath.from_subject\nbath from_subject","url":"bath.html#from_subject"},{"doc":"bath","title":"new","content":"pub fn new(\n  resource create_resource: fn() -&gt; Result(a, b),\n) -&gt; PoolConfig(a, b)\n Create a new [`PoolConfig`](#PoolConfig) for creating a pool of resources.\n\n \n import bath\n import fake_db\n\n pub fn main() {\n   // Create a pool of 10 connections to some fictional database.\n   let assert Ok(pool) =\n     bath.new(fn() { fake_db.connect() })\n     |&gt; bath.with_size(10)\n     |&gt; bath.start(1000)\n }\n \n\n ### Default values\n\n | Config | Default |\n |--------|---------|\n | `size`   | 10      |\n | `shutdown_resource` | `fn(_resource) { Nil }` |\n | `checkout_strategy` | `FIFO` |\n | `creation_strategy` | `Lazy` |\nSynonyms:\nbath.new\nbath new","url":"bath.html#new"},{"doc":"bath","title":"shutdown","content":"pub fn shutdown(\n  pool pool: Pool(a, b),\n  force force: Bool,\n  timeout timeout: Int,\n) -&gt; Result(Nil, ShutdownError)\n Shut down the pool, calling the `shutdown_function` on each\n resource in the pool. Calling with `force` set to `True` will\n force the shutdown, not calling the `shutdown_function` on any\n resources.\n\n Will fail if there are still resources checked out, unless `force` is\n `True`.\nSynonyms:\nbath.shutdown\nbath shutdown","url":"bath.html#shutdown"},{"doc":"bath","title":"start","content":"pub fn start(\n  config pool_config: PoolConfig(a, b),\n  timeout init_timeout: Int,\n) -&gt; Result(Pool(a, b), StartError(b))\n Start a  pool actor using the given [`PoolConfig`](#PoolConfig) and return a\n [`Pool`](#Pool).\nSynonyms:\nbath.start\nbath start","url":"bath.html#start"},{"doc":"bath","title":"with_checkout_strategy","content":"pub fn with_checkout_strategy(\n  config pool_config: PoolConfig(a, b),\n  strategy checkout_strategy: CheckoutStrategy,\n) -&gt; PoolConfig(a, b)\n Change the checkout strategy for the pool. Defaults to `FIFO`.\nSynonyms:\nbath.with_checkout_strategy\nbath with_checkout_strategy","url":"bath.html#with_checkout_strategy"},{"doc":"bath","title":"with_creation_strategy","content":"pub fn with_creation_strategy(\n  config pool_config: PoolConfig(a, b),\n  strategy creation_strategy: CreationStrategy,\n) -&gt; PoolConfig(a, b)\n Change the resource creation strategy for the pool. Defaults to `Lazy`.\nSynonyms:\nbath.with_creation_strategy\nbath with_creation_strategy","url":"bath.html#with_creation_strategy"},{"doc":"bath","title":"with_shutdown","content":"pub fn with_shutdown(\n  config pool_config: PoolConfig(a, b),\n  shutdown shutdown_resource: fn(a) -&gt; Nil,\n) -&gt; PoolConfig(a, b)\n Set a shutdown function to be run for each resource when the pool exits.\nSynonyms:\nbath.with_shutdown\nbath with_shutdown","url":"bath.html#with_shutdown"},{"doc":"bath","title":"with_size","content":"pub fn with_size(\n  config pool_config: PoolConfig(a, b),\n  size size: Int,\n) -&gt; PoolConfig(a, b)\n Set the pool size. Defaults to 10.\nSynonyms:\nbath.with_size\nbath with_size","url":"bath.html#with_size"},{"doc":"bath","title":"bath","content":"","url":"bath.html"}]);